@@ -4,6 +4,9 @@ use warnings;
 use strict;
 
 use HTML::Lint::Error;
+use HTML::Lint::Parser ();
+
+use HTML::Entities ();
 
 =head1 NAME
 
@@ -11,11 +14,11 @@ HTML::Lint - check for HTML errors in a string or file
 
 =head1 VERSION
 
-Version 2.10
+Version 2.12
 
 =cut
 
-our $VERSION = '2.10';
+our $VERSION = '2.12';
 
 =head1 SYNOPSIS
 
@@ -77,9 +80,8 @@ sub new {
 
     my $self = {
         _errors => [],
-        _types => [],
+        _types  => [],
     };
-    $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
     bless $self, $class;
 
     if ( my $only = $args{only_types} ) {
@@ -92,7 +94,24 @@ sub new {
     return $self;
 }
 
-=head2 $lint->parse( $chunk )
+=head2 $lint->parser()
+
+Returns the parser object for this object, creating one if necessary.
+
+=cut
+
+sub parser {
+    my $self = shift;
+
+    if ( not $self->{_parser} ) {
+        $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
+        $self->{_parser}->ignore_elements( qw(script style) );
+    }
+
+    return $self->{_parser};
+}
+
+=head2 $lint->parse( $text )
 
 =head2 $lint->parse( $code_ref )
 
@@ -104,7 +123,8 @@ See L<HTML::Parser>'s C<parse_file> method for details.
 
 sub parse {
     my $self = shift;
-    return $self->_parser->parse( @_ );
+
+    return $self->parser->parse( @_ );
 }
 
 =head2 $lint->parse_file( $file )
@@ -117,7 +137,7 @@ See L<HTML::Parser>'s C<parse_file> method for details.
 
 sub parse_file {
     my $self = shift;
-    return $self->_parser->parse_file( @_ );
+    return $self->parser->parse_file( @_ );
 }
 
 =head2 $lint->eof
@@ -134,9 +154,9 @@ sub eof {
     my $self = shift;
 
     my $rc;
-    my $parser = $self->_parser;
+    my $parser = $self->parser;
     if ( $parser ) {
-        $rc = $self->_parser->eof(@_);
+        $rc = $self->parser->eof(@_);
         delete $self->{_parser};
     }
 
@@ -218,10 +238,8 @@ in case, here you go.
 sub gripe {
     my $self = shift;
 
-    my $parser = $self->_parser;
-
     my $error = HTML::Lint::Error->new(
-        $self->{_file}, $parser->{_line}, $parser->{_column}, @_ );
+        $self->{_file}, $self->parser->{_line}, $self->parser->{_column}, @_ );
 
     my @keeps = @{$self->{_types}};
     if ( !@keeps || $error->is_type(@keeps) ) {
@@ -231,6 +249,7 @@ sub gripe {
     return;
 }
 
+
 =head2 $lint->newfile( $filename )
 
 Call C<newfile()> whenever you switch to another file in a batch
@@ -247,7 +266,6 @@ sub newfile {
     my $file = shift;
 
     delete $self->{_parser};
-    $self->{_parser} = HTML::Lint::Parser->new( sub { $self->gripe( @_ ) } );
     $self->{_file} = $file;
     $self->{_line} = 0;
     $self->{_column} = 0;
@@ -256,233 +274,45 @@ sub newfile {
     return $self->{_file};
 } # newfile
 
-sub _parser {
-    my $self = shift;
+1;
 
-    return $self->{_parser};
-}
+=head1 MODIFYING HTML::LINT'S BEHAVIOR
 
-=pod
+Sometimes you'll have HTML that for some reason cannot conform to
+HTML::Lint's expectations.  For those instances, you can use HTML
+comments to modify HTML::Lint's behavior.
 
-HTML::Lint::Parser is a class only for this module.
+Say you have an image where for whatever reason you can't get
+dimensions for the image.  This HTML snippet:
 
-=cut
+    <img src="logo.png" height="120" width="50" alt="Company logo">
+    <img src="that.png">
 
-package HTML::Lint::Parser;
+causes this error:
 
-use HTML::Parser 3.20;
-use HTML::Tagset 3.03;
+    foo.html (14:20) <img src="that.png"> tag has no HEIGHT and WIDTH attributes
 
-use HTML::Lint::HTML4 qw( %isKnownAttribute %isRequired %isNonrepeatable %isObsolete );
-use HTML::Entities qw( %char2entity );
+But if for some reason you can't get those dimensions when you build
+the page, you can at least stop HTML::Lint complaining about it.
 
-use base 'HTML::Parser';
+    <img src="this.png" height="120" width="50" alt="Company logo">
+    <!-- html-lint elem-img-sizes-missing: off, elem-img-alt-missing: off -->
+    <img src="that.png">
+    <!-- html-lint elem-img-sizes-missing: on, elem-img-alt-missing: off -->
 
-sub new {
-    my $class = shift;
-    my $gripe = shift;
+If you want to turn off all HTML::Lint warnings for a block of code, use
 
-    my $self =
-        HTML::Parser->new(
-            api_version => 3,
-            start_document_h   => [ \&_start_document, 'self' ],
-            end_document_h     => [ \&_end_document,   'self,line,column' ],
-            start_h            => [ \&_start,          'self,tagname,line,column,@attr' ],
-            end_h              => [ \&_end,            'self,tagname,line,column,tokenpos,@attr' ],
-            text_h             => [ \&_text,           'self,text' ],
-            strict_names       => 0,
-            empty_element_tags => 1,
-        );
-    bless $self, $class;
+    <!-- html-lint all: off -->
 
-    $self->{_gripe} = $gripe;
-    $self->{_stack} = [];
+And turn them back on with
 
-    return $self;
-}
+    <!-- html-lint all: off -->
 
-sub gripe {
-    my $self = shift;
+You don't have to use "on" and "off".  For "on", you can use "true"
+or "1".  For "off", you can use "0" or "false".
 
-    return $self->{_gripe}->( @_ );
-}
-
-sub _start_document {
-}
-
-sub _end_document {
-    my ($self,$line,$column) = @_;
-
-    for my $tag ( keys %isRequired ) {
-        if ( !$self->{_first_seen}->{$tag} ) {
-            $self->gripe( 'doc-tag-required', tag => $tag );
-        }
-    }
-
-    return;
-}
-
-sub _start {
-    my ($self,$tag,$line,$column,@attr) = @_;
-
-    $self->{_line} = $line;
-    $self->{_column} = $column;
-
-    my $validattr = $isKnownAttribute{ $tag };
-    if ( $validattr ) {
-        my %seen;
-        my $i = 0;
-        while ( $i < @attr ) {
-            my ($attr,$val) = @attr[$i++,$i++];
-            if ( $seen{$attr}++ ) {
-                $self->gripe( 'attr-repeated', tag => $tag, attr => $attr );
-            }
-
-            if ( $validattr && ( !$validattr->{$attr} ) ) {
-                $self->gripe( 'attr-unknown', tag => $tag, attr => $attr );
-            }
-        } # while attribs
-    }
-    else {
-        $self->gripe( 'elem-unknown', tag => $tag );
-    }
-    $self->_element_push( $tag ) unless $HTML::Tagset::emptyElement{ $tag };
-
-    if ( my $where = $self->{_first_seen}{$tag} ) {
-        if ( $isNonrepeatable{$tag} ) {
-            $self->gripe( 'elem-nonrepeatable',
-                            tag => $tag,
-                            where => HTML::Lint::Error::where( @{$where} )
-                        );
-        }
-    }
-    else {
-        $self->{_first_seen}{$tag} = [$line,$column];
-    }
-
-    # Call any other overloaded func
-    my $tagfunc = "_start_$tag";
-    if ( $self->can($tagfunc) ) {
-        $self->$tagfunc( $tag, @attr );
-    }
-
-    return;
-}
-
-sub _text {
-    my ($self,$text) = @_;
-
-    while ( $text =~ /([^\x09\x0A\x0D -~])/g ) {
-        my $bad = $1;
-        $self->gripe(
-            'text-use-entity',
-                char => sprintf( '\x%02lX', ord($bad) ),
-                entity => $char2entity{ $bad },
-        );
-    }
-
-    return;
-}
-
-sub _end {
-    my ($self,$tag,$line,$column,$tokenpos,@attr) = @_;
-
-    $self->{_line} = $line;
-    $self->{_column} = $column;
-
-    if ( !$tokenpos ) {
-        # This is a dummy end event for something like <img />.
-        # Do nothing.
-    }
-    elsif ( $HTML::Tagset::emptyElement{ $tag } ) {
-        $self->gripe( 'elem-empty-but-closed', tag => $tag );
-    }
-    else {
-        if ( $self->_in_context($tag) ) {
-            my @leftovers = $self->_element_pop_back_to($tag);
-            for ( @leftovers ) {
-                my ($tag,$line,$col) = @{$_};
-                $self->gripe( 'elem-unclosed', tag => $tag,
-                        where => HTML::Lint::Error::where($line,$col) )
-                        unless $HTML::Tagset::optionalEndTag{$tag};
-            } # for
-        }
-        else {
-            $self->gripe( 'elem-unopened', tag => $tag );
-        }
-    } # is empty element
-
-    # Call any other overloaded func
-    my $tagfunc = "_end_$tag";
-    if ( $self->can($tagfunc) ) {
-        $self->$tagfunc( $tag, $line );
-    }
-
-    return;
-}
-
-sub _element_push {
-    my $self = shift;
-    for ( @_ ) {
-        push( @{$self->{_stack}}, [$_,$self->{_line},$self->{_column}] );
-    } # while
-
-    return;
-}
-
-sub _find_tag_in_stack {
-    my $self = shift;
-    my $tag = shift;
-    my $stack = $self->{_stack};
-
-    my $offset = @{$stack} - 1;
-    while ( $offset >= 0 ) {
-        if ( $stack->[$offset][0] eq $tag ) {
-            return $offset;
-        }
-        --$offset;
-    } # while
-
-    return;
-}
-
-sub _element_pop_back_to {
-    my $self = shift;
-    my $tag = shift;
-
-    my $offset = $self->_find_tag_in_stack($tag) or return;
-
-    my @leftovers = splice( @{$self->{_stack}}, $offset + 1 );
-    pop @{$self->{_stack}};
-
-    return @leftovers;
-}
-
-sub _in_context {
-    my $self = shift;
-    my $tag = shift;
-
-    my $offset = $self->_find_tag_in_stack($tag);
-    return defined $offset;
-}
-
-# Overridden tag-specific stuff
-sub _start_img {
-    my ($self,$tag,%attr) = @_;
-
-    my ($h,$w,$src) = @attr{qw( height width src )};
-    if ( defined $h && defined $w ) {
-        # Check sizes
-    }
-    else {
-        $self->gripe( 'elem-img-sizes-missing', src=>$src );
-    }
-    if ( not defined $attr{alt} ) {
-        $self->gripe( 'elem-img-alt-missing', src=>$src );
-    }
-
-    return;
-}
+For a list of possible errors and their codes, see L<HTML::Lint::Error>,
+or run F<perldoc HTML::Lint::Error>.
 
 =head1 BUGS, WISHES AND CORRESPONDENCE
 
@@ -516,10 +346,10 @@ DO NOT send bug reports to http://rt.cpan.org/ or http://code.google.com/
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2011 Andy Lester.
+Copyright 2005-2012 Andy Lester.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the Artistic License v2.0.
+This program is free software; you can redistribute it and/or modify it
+under the terms of the Artistic License v2.0.
 
 http://www.opensource.org/licenses/Artistic-2.0
 
