@@ -6,24 +6,15 @@ use strict;
 use HTML::Parser 3.20;
 use HTML::Tagset 3.03;
 
-use HTML::Lint::Error ();
 use HTML::Lint::HTML4 qw( %isKnownAttribute %isRequired %isNonrepeatable %isObsolete );
-use HTML::Entities qw( %char2entity %entity2char );
+use HTML::Entities qw( %char2entity );
 
-use parent 'HTML::Parser';
+use base 'HTML::Parser';
 
 =head1 NAME
 
 HTML::Lint::Parser - Parser for HTML::Lint.  No user-serviceable parts inside.
 
-=head1 VERSION
-
-Version 2.27_03
-
-=cut
-
-our $VERSION = '2.27_03';
-
 =head1 SYNOPSIS
 
 See L<HTML::Lint> for all the gory details.
@@ -36,7 +27,8 @@ Constructor for the main parsing object.  The I<$gripe> argument
 is a coderef to a function that can handle errors from the parser.
 It is only ever (so far) C<HTML::Lint::gripe()>.
 
-=cut
+=cut=
+
 
 sub new {
     my $class = shift;
@@ -53,7 +45,6 @@ sub new {
             text_h             => [ \&_text,           'self,text' ],
             strict_names       => 0,
             empty_element_tags => 1,
-            attr_encoded       => 1,
         );
     bless $self, $class;
 
@@ -98,13 +89,12 @@ sub _displayable {
 }
 
 sub _start_document {
-    return;
 }
 
 sub _end_document {
     my ($self,$line,$column) = @_;
 
-    for my $tag ( sort keys %isRequired ) {
+    for my $tag ( keys %isRequired ) {
         if ( !$self->{_first_seen}->{$tag} ) {
             $self->gripe( 'doc-tag-required', tag => $tag );
         }
@@ -129,11 +119,9 @@ sub _start {
                 $self->gripe( 'attr-repeated', tag => $tag, attr => $attr );
             }
 
-            if ( !$validattr->{$attr} ) {
+            if ( $validattr && ( !$validattr->{$attr} ) ) {
                 $self->gripe( 'attr-unknown', tag => $tag, attr => $attr );
             }
-
-            $self->_entity($val, 'attr');
         } # while attribs
     }
     else {
@@ -165,56 +153,60 @@ sub _start {
 sub _text {
     my ($self,$text) = @_;
 
-    $self->_entity($text, 'text');
-
-    return;
-}
-
-sub _entity {
-    my ($self,$text,$type) = @_;
-
-    if ( not $self->{_entity_lookup} ) {
-        my @entities = sort keys %HTML::Entities::entity2char;
-        # Strip his semicolons
-        s/;$// for @entities;
-        $self->{_entity_lookup} = { map { ($_,1) } @entities };
+    while ( $text =~ /&(?![#0-9a-z])/ig ) {
+        $self->gripe( 'text-use-entity', char => '&', entity => '&amp;' );
     }
 
     while ( $text =~ /([^\x09\x0A\x0D -~])/g ) {
         my $bad = $1;
         $self->gripe(
-            $type . '-use-entity',
+            'text-use-entity',
                 char => sprintf( '\x%02lX', ord($bad) ),
-                entity => $char2entity{ $bad } || '&#' . ord($bad) . ';',
+                entity => $char2entity{ $bad },
         );
     }
 
-    while ( $text =~ /&([^ ;]*;?)/g ) {
-        my $match = $1;
+    if ( not $self->{_unclosed_entities_regex} ) {
+        # Get Gisle's list
+        my @entities = sort keys %HTML::Entities::entity2char;
 
-        if ( $match eq '' ) {
-            $self->gripe( $type . '-use-entity', char => '&', entity => '&amp;' );
-        }
-        elsif ( $match !~ m/;$/ ) {
-            if ( exists $self->{_entity_lookup}->{$match}
-                 || $match =~ m/^#(\d+)$/ || $match =~ m/^#x[\dA-F]+$/i) {
-                $self->gripe( $type . '-unclosed-entity', entity => "&$match;" );
+        # Strip his semicolons
+        s/;$// for @entities;
+
+        # Build a regex
+        my $entities = join( '|', @entities );
+        $self->{_unclosed_entities_regex} = qr/&($entities)(?!;)/;
+
+        $self->{_entity_lookup} = { map { ($_,1) } @entities };
+    }
+
+    while ( $text =~ m/$self->{_unclosed_entities_regex}/g ) {
+        my $ent = $1;
+        $self->gripe( 'text-unclosed-entity', entity => "&$ent;" );
+    }
+
+    while ( $text =~ m/&([^;]+);/g ) {
+        my $ent = $1;
+
+        # Numeric entities are fine, if they're not too large.
+        if ( $ent =~ /^#(\d+)$/ ) {
+            if ( $1 > 65536 ) {
+                $self->gripe( 'text-invalid-entity', entity => "&$ent;" );
             }
-            else {
-                $self->gripe( $type . '-unknown-entity', entity => "&$match" );
+            next;
+        }
+
+        # Hex entities are fine, if they're not too large.
+        if ( $ent =~ /^#x([\dA-F]+)$/i ) {
+            if ( length($1) > 4 ) {
+                $self->gripe( 'text-invalid-entity', entity => "&$ent;" );
             }
+            next;
         }
-        elsif ( $match =~ m/^#(\d+);$/ ) {
-            # All numeric entities are OK.  We used to check that they were in a given range.
-        }
-        elsif ( $match =~ m/^#x([\dA-F]+);$/i ) {
-            # All hex entities OK.  We used to check that they were in a given range.
-        }
-        else {
-            $match =~ s/;$//;
-            if ( !exists $self->{_entity_lookup}->{$match} ) {
-                $self->gripe( $type . '-unknown-entity', entity => "&$match;" );
-            }
+
+        # If it's not a numeric entity, then check the lookup table.
+        if ( !exists $self->{_entity_lookup}{$ent} ) {
+            $self->gripe( 'text-unknown-entity', entity => "&$ent;" );
         }
     }
 
@@ -234,7 +226,7 @@ sub _comment {
             my ($directive,$value) = split( /\s*:\s*/, $command, 2 );
             _trim($_) for ($directive,$value);
 
-            if ( ($directive ne 'all') &&
+            if ( ($directive ne 'all') && 
                 ( not exists $HTML::Lint::Error::errors{ $directive } ) ) {
                 $self->gripe( 'config-unknown-directive',
                     directive => $directive,
@@ -282,8 +274,8 @@ sub _normalize_value {
     my $what = shift;
 
     $what = _trim( $what );
-    return 1 if $what eq '1' || $what eq 'on'  || $what eq 'true';
-    return 0 if $what eq '0' || $what eq 'off' || $what eq 'false';
+    return 1 if $what =~ /^(?:1|on|true)$/;
+    return 0 if $what =~ /^(?:0|off|false)$/;
     return undef;
 }
 
@@ -294,7 +286,7 @@ sub _trim {
     return $_[0];
 }
 
-sub _end {  ## no critic ( Subroutines::ProhibitManyArgs ) I have no choice in what these args are.
+sub _end {
     my ($self,$tag,$line,$column,$tokenpos,@attr) = @_;
 
     $self->{_line} = $line;
@@ -377,7 +369,7 @@ sub _in_context {
 }
 
 # Overridden tag-specific stuff
-sub _start_img {    ## no critic ( Subroutines::ProhibitUnusedPrivateSubroutines )  # Called by parser based on tag name.
+sub _start_img {
     my ($self,$tag,%attr) = @_;
 
     my ($h,$w,$src) = @attr{qw( height width src )};
@@ -394,25 +386,4 @@ sub _start_img {    ## no critic ( Subroutines::ProhibitUnusedPrivateSubroutines
     return;
 }
 
-sub _start_input {  ## no critic ( Subroutines::ProhibitUnusedPrivateSubroutines )  # Called by parser based on tag name.
-    my ($self,$tag,%attr) = @_;
-
-    my ($type,$alt) = @attr{qw( type alt )};
-    if ( defined($type) && (lc($type) eq 'image') ) {
-        my $ok = defined($alt);
-        if ( $ok ) {
-            $alt =~ s/^ +//;
-            $alt =~ s/ +$//;
-            $ok = ($alt ne '');
-        }
-        if ( !$ok ) {
-            my $name = $attr{name};
-            $name = '' unless defined $name;
-            $self->gripe( 'elem-input-alt-missing', name => $name );
-        }
-    }
-
-    return;
-}
-
 1;
